#!/usr/bin/python3

"""

Example python script to extract and plot data from the MUWDemo model.

Remember to run this model first so you have the Results.sigurd output file!

"""

import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from matplotlib.backends.backend_pdf import PdfPages
from datetime import datetime
import math
import matplotlib.dates as mdates

days = mdates.DayLocator()


from volsung.volsungmodel import *

# load the model

model = VolsungModel("Results.sigurd")
pdfname = 'HistoryMatch.pdf'


pdf = PdfPages(pdfname)


zoneId = 0
wellnames = model.flowNetwork.portObjects
clr=['b','r','g','c','m','y','k', 'brown', 'mistyrose', 'indigo', 'olive'] # list of basic colors

#Plot well mass rates
for well in model.flowNetwork.portObjects:
    if well.name[0:3] == 'Pro':
        f = plt.figure()
        ax = plt.subplot(111)
        w_model = well.outputPorts[0].zoneHistory("Mass Rate")*3.6

        w_model[w_model == 0 ] = np.nan
        ax.plot(model.reservoir.toDateTime(model.reservoir.zoneTimes()), w_model, "o-b")
        w_data = well.outputPorts[0].fieldHistory("Mass Rate")*3.6
        w_data[w_data == 0 ] = np.nan
        ax.plot(model.reservoir.toDateTime(well.outputPorts[0].fieldHistory("Time")),w_data , "or")
        plt.ylim([0, 300
                  ])
        plt.ylabel("w [t/h]")
        plt.title(well.name)
        plt.legend(['Model', 'Data'], loc = 'best')   

        ax.xaxis.set_major_formatter(mdates.DateFormatter('%Y'))
        for tick in ax.get_xticklabels():
            tick.set_rotation(45)

        plt.savefig(pdf, format='pdf')
        
        plt.show()
        plt.close()

# output enthalpy over time
for well in model.flowNetwork.portObjects:
    if well.name[0:3] == 'Pro':
        
        f = plt.figure()
        ax = plt.subplot(111)
        h_model = well.outputPorts[0].zoneHistory("Enthalpy") /1e3
        h_model[h_model == 0 ] = np.nan
        ax.plot(model.reservoir.toDateTime(model.reservoir.zoneTimes()), h_model, "o-b")
        h_data = well.outputPorts[0].fieldHistory("Enthalpy")/1e3
        h_data[h_data == 0 ] = np.nan
        ax.plot(model.reservoir.toDateTime(well.outputPorts[0].fieldHistory("Time")),h_data , "or")
        plt.ylim([900, 1300])
        plt.ylabel("H [kJ/kg]")
        plt.title(well.name)
        plt.legend(['Model', 'Data'], loc = 'best')   
        #ax.xaxis.set_major_locator(days)
        ax.xaxis.set_major_formatter(mdates.DateFormatter('%Y'))
        for tick in ax.get_xticklabels():
            tick.set_rotation(45)
        #plt.savefig('Enthalpy' + well.name + '.png', format='png')
        plt.savefig(pdf, format='pdf')
        
        plt.show()
        plt.close()   

# Plot reservoir pressure over time
Phist = model.reservoir.history("/reservoir/Elements/Pressure")
### change the index by transposing it, so it is now indexed by element index, then time index
Phist = np.transpose(Phist)
## plot the pressure over time

f = plt.figure()
ax = plt.subplot(111)

pos = (3125,2625,-300) #Edit this specify a valid x, y, z coordinate    
cellid = model.reservoir.cellId(pos)
# now look up the element index for this cell and MINC layer
elementid = model.reservoir.elementIndex(cellid, 0)
# the pressure history for this cell
P = Phist[elementid]    # pressure in Pa, units are always strict SI

t = model.reservoir.zoneTimes()
# the times in simple fractional years
ty = model.reservoir.toYearFrac(t)
tdt = model.reservoir.toDateTime(t)


ax.plot(tdt, P/1e5,'-')

plt.ylabel("P [bar]")

plt.savefig(pdf, format='pdf')
plt.show()
plt.close()




pdf.close()
