#!/usr/bin/python3

"""

Example python script to extract and plot data from a Volsung model.

Remember to run this model first so you have the Results.sigurd output file!

"""

import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from matplotlib.backends.backend_pdf import PdfPages
from datetime import datetime
import vtk

from volsung.volsungmodel import *


# load the model
model = VolsungModel("Results.sigurd")


pdfname = 'NSMatch.pdf'
   

pdf = PdfPages(pdfname)
# plot reservoir temperature along a well track, including field data
zoneId = 0 #This will take the last time zone in the output file.  Use 0 to get the first time

wellnames = model.flowNetwork.portObjectNames()
wellnames = ['Prod1', 'Prod2']


# plot a temperature slice through the reservoir
# and display the welltracks on it

# create the slice as a background

zs = [50]
for z in zs:  
    print (z)
    f = plt.figure()
    ax = plt.subplot(111)                                       # last zone
    model.reservoir.slicePlot(f, ax, z, zoneId, arrname = "Temperature", offset = -273.15)
    # now print the welltracks onto the plot
    model.flowNetwork.plotWellTracks(f, ax, linewidth = 1, color = '#000000', fontsize = 0)
    
    # plot a temperature slice through the reservoir
    # where the welltracks cross the slice plot discs with well field data
    # create the slice as a background
    #model.reservoir.slicePlot(f, ax, z, zoneid, arrname = "Temperature", offset = -273.15)
    # now print the welltrack intersections onto the plot
    for wellname in wellnames:
        well = model.flowNetwork.portObject(wellname)
    
        well.plotWellTrackFieldData(f, ax, z, float('nan'), "Temperature", offset = -273.15, radius = 120.0, fontsize = 0, labeloffset = (50, 50))
    ax.axis('equal')    # set aspect ratio to equal
    
    # show the plots
    plt.title('z = '+ str(z) + 'm')
    plt.xticks(rotation=30)
    #plt.savefig('slice' + str(z) + '.png', format='png')
    plt.savefig(pdf, format='pdf')
    plt.show()
    plt.close()


for wellname in wellnames:
    monitor = model.flowNetwork.portObject(wellname)

    kernel = vtk.vtkShepardKernel()
    kernel.SetNumberOfPoints(12)
    kernel.SetKernelFootprintToNClosest()
    interp = model.reservoir.interpolateWellTrack(monitor.wellGridGeometry[zoneId], zoneId, kernel = kernel)
    probe = model.reservoir.probeWellTrack(monitor.wellGridGeometry[zoneId], zoneId, average = False)
    
    ## plot the field data by requiring a time slice.
    ## if time is nan then all data is returned; else data closest to the time provided is chosen
    fielddata = monitor.reservoir1DFieldData.timeSlice(time = float('nan'))
    
    plt.figure()
    plt.plot(interp["Temperature"] - 273.15, interp["z"], '-b')
    #plt.plot(probe["Temperature"] - 273.15, probe["z"], '--b')
    plt.plot(fielddata["Temperature"] - 273.15, fielddata["z"], "-r")
    
    plt.xlim(0,350)
    #plt.ylim(-100,600)
    plt.ylabel("z [m]")
    plt.xlabel("Temperature [C]")
    plt.legend(['Model','Measured'])
    plt.title(wellname + ' temperature')
    plt.savefig(pdf, format='pdf')
    #plt.savefig('temperature' + wellname + '.png', format='png')
    plt.show()
    plt.close()

##Example of natural state pressure plot
#zs = [-200, -1600]
## Insert natural state pressure gradient from data 
#ns_p = (np.array(zs) - 303.07719571162966)/-12.347762839762611
#plt.figure()
#for wellname in wellnames:
#
#    kernel = vtk.vtkShepardKernel() # JC to test difference kernels to decide on default
#
#    kernel.SetNumberOfPoints(32)
#    kernel.SetKernelFootprintToNClosest()
#            
#    monitor = model.flowNetwork.portObject(wellname)
#    interp = model.reservoir.interpolateWellTrack(monitor.wellGridGeometry[zoneId], zoneId, kernel = kernel)
#    probe = model.reservoir.probeWellTrack(monitor.wellGridGeometry[zoneId], zoneId, average = False)
#    ## if time is nan then all data is returned; else data closest to the time provided is chosen
#    fielddata = monitor.reservoir1DFieldData.timeSlice(time = float('nan'))    
#    plt.plot(interp["Pressure"]/1e5, interp["z"], '--b', alpha = 0.3)
#
#plt.plot(ns_p, zs, 'r-', alpha = 1, linewidth = 2)
#plt.grid()
#plt.ylabel("z [m]")
#plt.xlabel("Pressure [bar]")
#plt.title("Natural State Pressure")
#plt.savefig(pdf, format='pdf')
#
#plt.show()
plt.close()




pdf.close()
